<?php

namespace Teknisa\FAI\Modules\Apuracao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\GER\Utilities\Environment;
use \Teknisa\Model\Geral\Entities\Base\Ctrlapurfat;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Fai04200ControleVendaAntecipada extends \Teknisa\GER\Controllers\TeknisaCrud {
    const GESTAO_DIRETA = '001';
    const PARAMBLOQ = 'fai04200_parambloqueio';

    protected $environment;
    protected $crudService;
    protected $parameterBag;
    protected $dataSourceName;
    protected $parameterBagColumns;
    protected $procedureApuracGesdireta;
    protected $procedureGeraDevolveAntecipada;
    protected $procedureDeletaDevolveAntecipada;
    protected $newCode;
    protected $dataSourceManager;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $environment, StoredProcedure $procedureApuracGesdireta, StoredProcedure $procedureGeraDevolveAntecipada, StoredProcedure $procedureDeletaDevolveAntecipada, $newCode, $dataSourceManager, $entityManager) {
        $this->newCode = $newCode;
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->procedureApuracGesdireta = $procedureApuracGesdireta;
        $this->procedureGeraDevolveAntecipada = $procedureGeraDevolveAntecipada;
        $this->procedureDeletaDevolveAntecipada = $procedureDeletaDevolveAntecipada;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager = $entityManager;
    }

    public function findServicoClienteFilial(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'fai04200_servico_cliente_filial';
        $this->parameterBagColumns = array("CDFILIAL", "CDCLIENTE");
        parent::find($request, $response);
    }

    public function findClient(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->addCondition('P_CDFILIAL', "=", $this->environment->getCdFilial());
        $this->dataSourceName = "clienfilial_cliente_cva";
        $this->parameterBagColumns = array("P_CDFILIAL", "P_CDCLIENTE");
        parent::find($request, $response);
    }

    public function findClientService(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->addCondition('P_CDFILIAL', "=", $this->environment->getCdFilial());
        $this->dataSourceName = "servclie_servico";
        $this->parameterBagColumns = array("P_CDFILIAL", "P_CDCLIENTE", "P_CDSERVFILI");
        parent::find($request, $response);
    }

    public function findServiceClient(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->addCondition('P_CDFILIAL', "=", $this->environment->getCdFilial());
        $this->dataSourceName = "servico_servclie";
        $this->parameterBagColumns = array("P_CDFILIAL", "P_CDCLIENTE");
        parent::find($request, $response);
    }

    public function findService(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $this->dataSourceName = "fai04200_ctrlvendaantec_cliente_servico_notafiscal";
        $this->parameterBagColumns = array("CDFILIAL", "CDCLIENTE", "DATAINICIAL", "DATAFINAL");
        parent::find($request, $response);
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = "ctrlvendaantec";
        parent::delete($request, $response);
    }

    private function inserirControleApuracaoFaturamento($cdfilial, $nrapuracfat, $cdcliente, $dtiniapurac, $dtfimapurac, $nrorg) {
        try{
            $ctrlapurfat = new \Geral\Entities\Ctrlapurfat();
            $ctrlapurfat->setCdfilial($cdfilial);
            $ctrlapurfat->setNrapuracfat($nrapuracfat);
            $ctrlapurfat->setCdcliente($cdcliente);
            $ctrlapurfat->setDtiniapurac($dtiniapurac);
            $ctrlapurfat->setDtfimapurac($dtfimapurac);
            $ctrlapurfat->setCdfiliapur($cdfilial);
            $ctrlapurfat->setCdtipofatuct(self::GESTAO_DIRETA);
            $ctrlapurfat->setNrorg($nrorg);
            $ctrlapurfat->setDtinclusao(new \DateTime());
            $ctrlapurfat->setDtultatu(new \DateTime());
            $ctrlapurfat->setCdoperinclusao($this->environment->getCdoperador());
    
            $this->entityManager->persist($ctrlapurfat);
            $this->entityManager->flush();
        }catch(\Exception $e){
            throw $e;
        }

    }

    private function atualizarControleApuracaoFaturamento($cdfilial, $nrapuracfat, $cdcliente, $dtiniapurac, $dtfimapurac, $nrorg) {
        try{
            $ctrlapurfat = $this->entityManager->getRepository('Geral\Entities\Ctrlapurfat')->findOneBy(array(
                'cdfilial' => $cdfilial,
                'nrorg' => $nrorg,
                'nrapuracfat' => $nrapuracfat,
                'cdcliente' => $cdcliente
            ));
    
            $ctrlapurfat->setDtiniapurac($dtiniapurac);
            $ctrlapurfat->setDtfimapurac($dtfimapurac);
    
            $this->entityManager->persist($ctrlapurfat);
            $this->entityManager->flush();
        }catch(\Exception $e){
            throw $e;
        }

    }

    private function atualizaControleDeVendaAntecipada($servicos, $cdfilial, $nrapuracfat, $cdcliente, $dtiniapurac, $nrorgtrab) {
        try{
            //@todo - temporario. Trocar para entidade.
            foreach ($servicos as $servico) {
                $stmt = $this->entityManager->getConnection()->prepare("UPDATE CTRLVENDAANTEC
                 SET
                   NRAPURACFAT= :P_NRAPURACFAT
                 WHERE
                   CDFILIAL   = :P_CDFILIAL   AND
                   CDCLIENTE  = :P_CDCLIENTE  AND
                   CDSERVFILI = :P_CDSERVFILI AND
                   NRORG = :P_NRORG AND
                   TRUNC(DTVENDA)    = :P_DTVENDA");
    
                $stmt->execute(
                    array(
                        'P_CDFILIAL' => $cdfilial,
                        'P_CDCLIENTE' => $cdcliente,
                        'P_CDSERVFILI' => $servico,
                        'P_DTVENDA' => $dtiniapurac->format('d/m/Y'),
                        'P_NRAPURACFAT' => $nrapuracfat,
                        'P_NRORG' => $nrorgtrab
                    )
                );
            }
        }catch(\Exception $e){
            throw $e;
        }
    }

    private function retornaNumeroLancamentoNotaFiscal($nrapuracfat, $cdcliente, $cdfilial, $nrorgtrab) {

        $notafiscal = $this->entityManager->getRepository('Fiscal\Entities\Notafiscal')->findOneBy(array(
            'cdfilial' => $cdfilial,
            'nrorg' => $nrorgtrab,
            'nrapuracfat' => $nrapuracfat,
            'cdcliente' => $cdcliente
        ));
        
        return $notafiscal->getNrlanctonf();

    }
    private function getServices($rows) {
        $services = array();
        foreach ($rows as $row) {
            $services[] = $row["CDSERVFILI"];
        }
        return $services;
    }

    public function calculoVendaAntecipada(DTO\Request\Row $request, DTO\Response $response){
        try {
            $data = $request->getRow();
            $rows = $data["rows"];
            $mensagem = isset($data['ADDITIONALDATATEXT']) ? $data['ADDITIONALDATATEXT'] : "";
            
            $cdfilial = $data['CDFILIAL'];
            $nrorgtrab = $this->environment->getNrOrg();
            $dtvenda = \DateTime::createFromFormat('d/m/Y', explode(' ', $rows[0]['DTVENDA'])[0]);
            
            $cdcliente = $rows[0]['CDCLIENTE'];
            $services = $this->getServices($rows);
            $joinServices = implode('_', $services) ;
            $nrapuracfat = empty($row['NRAPURACFAT']) ? null : $row['NRAPURACFAT'];
            $cdtipooper = null;
            if (isset($nrapuracfat) && $nrapuracfat != null) {
                $this->atualizarControleApuracaoFaturamento($cdfilial, $nrapuracfat, $cdcliente, $dtvenda, $dtvenda, $nrorgtrab);
            } else {
                $contador = 'APURACFATUR'.$cdfilial;
                $nrapuracfat = $this->newCode->getCodeByTableName($contador, $nrorgtrab);
                $this->inserirControleApuracaoFaturamento($cdfilial, $nrapuracfat, $cdcliente, $dtvenda, $dtvenda, $nrorgtrab);
                $this->atualizaControleDeVendaAntecipada($services, $cdfilial, $nrapuracfat, $cdcliente, $dtvenda, $nrorgtrab);
            }
            
            $this->procedureApuracGesdireta->addParam(new Param('P_CDFILIAL',   Param::PARAM_INPUT, $cdfilial,                  Param::PARAM_TYPE_STR));
            $this->procedureApuracGesdireta->addParam(new Param('P_CDCLIENTE',  Param::PARAM_INPUT, $cdcliente,                 Param::PARAM_TYPE_STR));
            $this->procedureApuracGesdireta->addParam(new Param('P_DATAINI',    Param::PARAM_INPUT, $dtvenda->format('d/m/Y'),  Param::PARAM_TYPE_STR));
            $this->procedureApuracGesdireta->addParam(new Param('P_DATAFIM',    Param::PARAM_INPUT, $dtvenda->format('d/m/Y'),  Param::PARAM_TYPE_STR));
            $this->procedureApuracGesdireta->addParam(new Param('P_NRAPURACFAT',Param::PARAM_INPUT, $nrapuracfat,               Param::PARAM_TYPE_STR));
            $this->procedureApuracGesdireta->addParam(new Param('P_IDGERANOTA', Param::PARAM_INPUT, 'S',                        Param::PARAM_TYPE_STR));
            $this->procedureApuracGesdireta->addParam(new Param('P_DTSAIDAPUR', Param::PARAM_INPUT, $dtvenda->format('d/m/Y'),  Param::PARAM_TYPE_STR));
            $this->procedureApuracGesdireta->addParam(new Param('P_CDTIPOOPER', Param::PARAM_INPUT, $cdtipooper,                Param::PARAM_TYPE_STR));
            $this->procedureApuracGesdireta->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $joinServices,              Param::PARAM_TYPE_STR));
            $this->procedureApuracGesdireta->execute();

            $nrlanctonf = $this->retornaNumeroLancamentoNotaFiscal($nrapuracfat, $cdcliente, $cdfilial, $nrorgtrab);
            if($mensagem!=''){
                $this->callProcedureAdditionalData($cdfilial, $nrlanctonf, $dtvenda, $mensagem);
            }

            $response->addDataSet(new DataSource\DataSet ('notafiscal', array("nrlanctonf" => $nrlanctonf, "nrapuracfat" => $nrapuracfat)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function callProcedureAdditionalData($cdfilial, $nrlanctonf, $dtvenda, $mensagem) {
        try{
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_NOTAFISCAL_DADOSADIC');
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $dtvenda->format('d/m/Y')));
            $procedure->addParam(new Param('P_DATAFIM', Param::PARAM_INPUT, $dtvenda->format('d/m/Y')));
            $procedure->addParam(new Param('P_MENSAGEM', Param::PARAM_INPUT, $mensagem, Param::PARAM_TYPE_STR));
            $procedure->execute();
        }catch(\Exception $e){
            throw $e;
        }
    }

    private function getParambloq($cdfilial, $nrorg) {
        $filterCriteria = new FilterCriteria(self::PARAMBLOQ);
        $this->parameterBagColumns = array('NRORG', 'CDFILIAL');
        $filterCriteria->addCondition('NRORG', $nrorg);
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        return $this->dataSourceManager->findBy($newFilterCriteria);
    }

    private function validateDate($cdfilial, $nrorg, $dtvenda) {
        $message = false;
        $parambloq = $this->getParambloq($cdfilial, $nrorg)->getRows();
        
        
		$this->parameterBagColumns = array();
		$filterCriteria = new FilterCriteria('paramgeral');
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		$paramgeral = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
        
        
        if (!empty($parambloq)) {

            if($paramgeral[0]['IDUTILDTPROC']=='S'){
                $dtprocessa = \DateTime::createFromFormat('d/m/Y H:i:s', $parambloq[0]['DTPROCESSA']);
            } else {
                $dtprocessa = \DateTime::createFromFormat('d/m/Y H:i:s', (new \DateTime())->format('d/m/Y H:i:s'));
            }
            
            
            $dtvenda = \DateTime::createFromFormat('d/m/Y H:i:s', $dtvenda.'00:00:00');
            $idlancdtmen = $parambloq[0]['IDLANCDTMEN'];
            $idlancdtmai = $parambloq[0]['IDLANCDTMAI'];
            $nrdiassaid = $parambloq[0]['NRDIASSAID'] ? $parambloq[0]['NRDIASSAID'] : 0;
            $nrdiassaidpd = $parambloq[0]['NRDIASSAIDPD'] ? $parambloq[0]['NRDIASSAIDPD'] : 0;
            $idlancdtmenfil = $parambloq[0]['IDLANCDTMENFIL'];
            $nrdiassaidfil = $parambloq[0]['NRDIASSAIDFIL'] ? $parambloq[0]['NRDIASSAIDFIL'] : 0;
            $idlancdtmaifil = $parambloq[0]['IDLANCDTMAIFIL'];
            $nrdiassaidpdfil = $parambloq[0]['NRDIASSAIDPDFIL'] ? $parambloq[0]['NRDIASSAIDPDFIL'] : 0;

            if ($dtvenda < $dtprocessa) {
                if ($idlancdtmenfil == 'N') {
                    if ($idlancdtmen == 'N') {
                        $message = "dateLessThanProc";
                    } else {
                        if( $dtvenda < ($dtprocessa->sub(new \DateInterval('P'.$nrdiassaid.'D')))) {
                            $message = "dateOutOfPeriod";
                        }
                    }
                } else {
                    if( $dtvenda < ($dtprocessa->sub(new \DateInterval('P'.$nrdiassaidfil.'D')))) {
                        $message = "dateOutOfPeriod";
                    }
                }
            } else if ($dtvenda > $dtprocessa) {
                if ($idlancdtmaifil == 'S') {
                    if( $dtvenda > ($dtprocessa->add(new \DateInterval('P'.$nrdiassaidpdfil.'D')))) {
                        $message = "dateOutOfPeriod";
                    }
                } else {
                    if ($idlancdtmai == 'S') {
                        if ( $dtvenda > ($dtprocessa->add(new \DateInterval('P'.$nrdiassaidpd.'D')))) {
                            $message = "dateOutOfPeriod";
                        }
                    } else {
                        $message = "dateGreaterThanProc";
                    }
                }
            }
        }
        return $message;
    }
    
    public function getParamServRef(DTO\Request\Row $request, DTO\Response $response) {
        $sql = 'SELECT 
                    MAX(SUBSTR(DSPARAMFATGE,11,1)) AS PARAMSERVREF
                FROM PARAMGERAL';
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute();
        $param = $query->fetch()['PARAMSERVREF'];
        $response->addDataSet(new DataSource\DataSet ('notafiscal', array("PARAMSERVREF" => $param)));
    }

    public function saveCtrlvendaantec(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $cdfilial = $this->environment->getCdFilial();
            $nrorg = $this->environment->getNrOrg();
            $row = $request->getRow();
            $dtvenda = $row["DTVENDA"];

            $invalidDate = $this->validateDate($cdfilial, $nrorg, $dtvenda);
            
            if ($invalidDate) {
                throw new \Exception($invalidDate);
            } else {
                $this->dataSourceName = "ctrlvendaantec";
                $exists = $this->verifyCtrlVend($row);
                if($exists){
                    throw new \Exception('existsCtrlVend');
                }
                parent::save($request, $response);
            }

        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    private function verifyCtrlVend($row){
        $sql = 'SELECT
                    CT.CDFILIAL,
                    CT.CDCLIENTE,
                    CT.CDSERVFILI,
                    CT.DTVENDA
                FROM
                    CTRLVENDAANTEC CT
                WHERE
                    ( CT.CDFILIAL =:CDFILIAL )
                    AND ( CT.CDCLIENTE =:CDCLIENTE )
                    AND ( CT.CDSERVFILI =:CDSERVFILI )
                    AND ( TRUNC(CT.DTVENDA) =:DTVENDA )';
        $params = array(
            'CDFILIAL'   => $row['CDFILIAL'],
            'CDCLIENTE'  => $row['CDCLIENTE'],
            'CDSERVFILI' => $row['CDSERVFILI'],
            'DTVENDA'    => $row['DTVENDA']
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $rows = $query->fetchAll();
        if(empty($rows)){
            return false;
        }
        return true;
    }
    
    public function geraDevolvidaAntec(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $this->entityManager->getConnection()->beginTransaction();
            
            $this->procedureGeraDevolveAntecipada->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $row['P_CDFILIAL'],                Param::PARAM_TYPE_STR));
            $this->procedureGeraDevolveAntecipada->addParam(new Param('P_CDCLIENTE',   Param::PARAM_INPUT, $row['P_CDCLIENTE'],               Param::PARAM_TYPE_STR));
            $this->procedureGeraDevolveAntecipada->addParam(new Param('P_CDSERVFILI',  Param::PARAM_INPUT, $row['P_CDSERVFILI'],              Param::PARAM_TYPE_STR));
            $this->procedureGeraDevolveAntecipada->addParam(new Param('P_QTDE',        Param::PARAM_INPUT, $row['P_QTDE'],                    Param::PARAM_TYPE_STR));
            $this->procedureGeraDevolveAntecipada->addParam(new Param('P_VRSERVICO',   Param::PARAM_INPUT, $row['P_VRSERVICO'],               Param::PARAM_TYPE_STR));
            $this->procedureGeraDevolveAntecipada->addParam(new Param('P_DTDEVOLUCAO', Param::PARAM_INPUT, $row['P_DTDEVOLUCAO'],             Param::PARAM_TYPE_STR));
            $this->procedureGeraDevolveAntecipada->addParam(new Param('P_CDTIPOOPER',  Param::PARAM_INPUT, $row['P_CDTIPOOPER'],              Param::PARAM_TYPE_STR));
            $this->procedureGeraDevolveAntecipada->addParam(new Param('P_CDOPERADOR',  Param::PARAM_INPUT, $row['P_CDOPERADOR'],              Param::PARAM_TYPE_STR));
            $this->procedureGeraDevolveAntecipada->addParam(new Param('P_NRORG',       Param::PARAM_INPUT, $row['P_NRORG'],                   Param::PARAM_TYPE_STR));
            $this->procedureGeraDevolveAntecipada->addParam(new Param('P_CDFILIALORIG',Param::PARAM_INPUT, $row['P_CDFILIAL'],                Param::PARAM_TYPE_STR));
            $this->procedureGeraDevolveAntecipada->addParam(new Param('P_NRLANCTOORIG',Param::PARAM_INPUT, $row['P_NRLANCTOORIG'],            Param::PARAM_TYPE_STR));
            $this->procedureGeraDevolveAntecipada->execute();
            
            $this->entityManager->getConnection()->commit();
            $response->addDataSet(new DataSource\DataSet ('GERA_DEVOLVEATENC', array("MSG" => 1)));
        }catch (\Exception $e) {
            $this->entityManager->getConnection()->rollback();
            $msg = $e->getMessage();
			if($e->getErrorCode() == 20000){
			    $array_messages = explode("\n",$e->getMessage());
				$size = count($array_messages);
				if($size >= 3){
					$parts_message = explode("ORA-20000: ",$array_messages[2]);
					$size_parts = count($parts_message);
					if($size_parts >= 2){
						$msg = utf8_decode($parts_message[1]);
					}
				}
			    $response->addDataSet(new DataSource\DataSet ('GERA_DEVOLVEATENC', array("MSG" => $msg)));
			}else{
			    $response->setError(new Error($msg, 500));
			}
			
        }
    }
    
    public function deletaDevolvidaAntec(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $this->entityManager->getConnection()->beginTransaction();
            
            $this->procedureDeletaDevolveAntecipada->addParam(new Param('P_NRDEVOLVENDAANTEC',   Param::PARAM_INPUT, $row['P_NRDEVOLVENDAANTEC'],                Param::PARAM_TYPE_STR));
            $this->procedureDeletaDevolveAntecipada->addParam(new Param('P_CDOPERADOR',          Param::PARAM_INPUT, $row['P_CDOPERADOR'],                       Param::PARAM_TYPE_STR));
            $this->procedureDeletaDevolveAntecipada->addParam(new Param('P_NRORG',               Param::PARAM_INPUT, $row['P_NRORG'],                            Param::PARAM_TYPE_STR));
            $this->procedureDeletaDevolveAntecipada->addParam(new Param('P_ERRO',                Param::PARAM_OUTPUT,  null,                                     Param::PARAM_TYPE_STR, 30));
            $res = $this->procedureDeletaDevolveAntecipada->execute();
            $this->entityManager->getConnection()->commit();
            $response->addDataSet(new DataSource\DataSet ('DELETA_DEVOLVEATENC', $res));
        }catch (\Exception $e){
			$this->entityManager->getConnection()->rollback();
			$response->setError(new Error($e->getMessage(), 500));
        }
    }
}