<?php
namespace Teknisa\FAI\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Fai03600RelatorioCustoDeRefeicoes extends \Zeedhi\Framework\Controller\Report {

    protected $newCode;
    protected $nrorg;
    protected $cdfilial;
    protected $qr2Strategy;
    protected $procedureMovimentacaoEstoque;
    protected $procedureApuracRelgesdireta;
    protected $environment;
    protected $connection;

    protected $deleteVar;
    protected $deleteVar2;

    public function __construct(ReportService $reportService, $newCode, $qr2Strategy, StoredProcedure $procedureMovimentacaoEstoque, StoredProcedure $procedureApuracRelgesdireta, $environment, $connection) {
        $this->newCode                      = $newCode;
        $this->qr2Strategy                  = $qr2Strategy;
        $this->procedureMovimentacaoEstoque = $procedureMovimentacaoEstoque;
        $this->procedureApuracRelgesdireta  = $procedureApuracRelgesdireta;
        $this->environment                  = $environment;
        $this->connection                   = $connection;
        parent::__construct($reportService);
    }

    protected $reportMapping = array (
        'fai03601' =>
        array (
            'strategy' => 'QR2',
            'parameterMapping' =>
            array (
                "CDFILIAL"   => "P_CDFILIAL",
                "DTINICIAL"  => "P_DTINICIAL",
                "DTFINAL"    => "P_DTFINAL",
                "NRSEQUMOVI" => "P_NRSEQUMOVI",
                "NRAPURACGD" => "P_NRAPURACGD",
                "CDPRODINI"  => "P_CDPRODINI",
                "CDPRODFIN"  => "P_CDPRODFIN"
            ),
        ),
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {

        $row = $request->getRow();

        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);

        $filial = $row['CDFILIAL'];
        $cdalmoxarife = ' ';
        $cdlocalestoq = ' ';
        $nrloteestq = ' ';
        $tamnivel = 2;
        $dtinicial = $row['DTINICIAL'];
        $dtfinal = $row['DTFINAL'];
        $prodini = $row['CDARVPROD'];
        $prodfim = str_pad($row['CDARVPROD2'], 10, "z", STR_PAD_RIGHT);
        $idlancto = 1;
        $idcusto = 1;
        $this->deleteVar = $this->newCode->getCodeByTableName('MOVIESTOQUE', $this->environment->getNrOrgTrab());
        $movconsolid = null;
        $custoalmox = null;
        $delmovimen = null;

        $this->procedureMovimentacaoEstoque->addParam(new Param('P_FILIAL', Param::PARAM_INPUT, $filial));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_CDALMOXARIFE', Param::PARAM_INPUT, $cdalmoxarife));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_CDLOCALESTOQ', Param::PARAM_INPUT, $cdlocalestoq));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_NRLOTEESTQ', Param::PARAM_INPUT, $nrloteestq));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_TAMNIVEL', Param::PARAM_INPUT, $tamnivel));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $dtinicial));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $dtfinal));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_PRODINI', Param::PARAM_INPUT, $prodini));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_PRODFIM', Param::PARAM_INPUT, $prodfim));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_IDLANCTO', Param::PARAM_INPUT, $idlancto));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_IDCUSTO', Param::PARAM_INPUT, $idcusto));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_SEQUMOVI', Param::PARAM_INPUT, $this->deleteVar));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_MOVCONSOLID', Param::PARAM_INPUT, $movconsolid));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_CUSTOALMOX', Param::PARAM_INPUT, $custoalmox));
        $this->procedureMovimentacaoEstoque->addParam(new Param('P_DELMOVIMEN', Param::PARAM_INPUT, $delmovimen));

        $this->procedureMovimentacaoEstoque->execute();

        $cdfilial = $row['CDFILIAL'];
        $dataini = $row['DTINICIAL'];
        $datafin = $row['DTFINAL'];
        $this->deleteVar2 = $this->newCode->getCodeByTableName('RELAPURFAT', $this->environment->getNrOrgTrab());
        $cdcliente = null;

        $this->procedureApuracRelgesdireta->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial));
        $this->procedureApuracRelgesdireta->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $dataini));
        $this->procedureApuracRelgesdireta->addParam(new Param('P_DATAFIN', Param::PARAM_INPUT, $datafin));
        $this->procedureApuracRelgesdireta->addParam(new Param('P_NRAPURACGD', Param::PARAM_INPUT, $this->deleteVar));
        $this->procedureApuracRelgesdireta->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $cdcliente));

        $this->procedureApuracRelgesdireta->execute();
        
        $row['DTINICIAL'] = $row['DTINICIAL'];
        $row['DTFINAL'] = $row['DTFINAL'];
        $row['NRSEQUMOVI'] = $this->deleteVar;
        $row['NRAPURACGD'] = $this->deleteVar2;
        $row['CDPRODINI'] = $row['CDARVPROD'];
        $row['CDPRODFIN'] = $row['CDARVPROD2'];
    }

    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {

        $deleteRelatMoviest = $this->connection->prepare('DELETE FROM RELAT_MOVIEST WHERE (NRRELATMOVI = :P_MOVIESTOQUE)');
        $deleteRelatDescmovi = $this->connection->prepare('DELETE FROM RELAT_DESCMOVI WHERE (NRRELATDESC = :P_MOVIESTOQUE)');
        $deleteRelatTotamovi = $this->connection->prepare('DELETE FROM RELAT_TOTAMOVI WHERE (NRRELATTOTA = :P_MOVIESTOQUE)');
        $deleteTrabApuracGd = $this->connection->prepare('DELETE TRAB_APURAC_GD WHERE (CDFILIALAPGD = :P_CDFILIAL) AND (NRAPURACGD   = :P_NRAPURACGD)');

        $deleteRelatMoviest->execute(array('P_MOVIESTOQUE' => $this->deleteVar));
        $deleteRelatDescmovi->execute(array('P_MOVIESTOQUE' => $this->deleteVar));
        $deleteRelatTotamovi->execute(array('P_MOVIESTOQUE' => $this->deleteVar));
        $deleteTrabApuracGd->execute(array('P_CDFILIAL' => $this->deleteVar2, 'P_NRAPURACGD'=> $this->deleteVar2));

    }

}